<?php

/**
 * @package         Convert Forms
 * @version         5.0.0 Free
 * 
 * @author          Tassos Marinos <info@tassos.gr>
 * @link            https://www.tassos.gr
 * @copyright       Copyright © 2025 Tassos All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 */

namespace ConvertForms\Validation\Rules;

defined('_JEXEC') or die('Restricted access');


/**
 * Validation Rule - Time To Submit
 * 
 * This rule enforces a minimum time requirement between form display and submission
 * to help prevent automated bot submissions. If a form is submitted too quickly,
 * it's likely from a bot rather than a human user.
 */
class TimeToSubmit extends \ConvertForms\Validation\Rule
{
    // Returns the session namespace for the form's initial timestamp
    private $sessionProp = 'convertforms.initTime';

    /**
     * Validates the time elapsed between form display and submission
     *
     * @return boolean Returns false if submission is too quick, true otherwise
     */
    public function validate()
    {   
        // Get minimum required time in seconds
        $minTime = $this->getMinTimeToSubmit();

        if ($minTime === 0)
        {
            return true;
        }
        
        // Get the timestamp when the form was initially displayed
        $startTime = $this->getStartTime();

        // Reset the timestamp to force a new waiting period after each attempt.
        // This serves two purposes:
        // 1. Prevents rapid-fire submissions from bots
        // 2. Prevents multiple submissions from the same user in case of a slow server response
        $this->setStartTime();

        // Calculate elapsed time in seconds
        $timeElapsed = time() - $startTime;

        // Check if submission is too quick (likely a bot)
        if (is_null($startTime) || $timeElapsed < $minTime)
        {
            return false;
        }
    }

    public function onFormBeforeRender()
    {
        if ($this->getMinTimeToSubmit() > 0)
        {
            $boxAtts = isset($this->form['containerAtts']) ? $this->form['containerAtts'] : [];

            // Let's add some data attributes for the client-side script.
            $this->form['containerAtts'] = array_merge($boxAtts, [
                // Fire the session XHR request which fires the onInit() method
                'data-cf-ss' => '',
            ]);
        }
    }

    /**
     * Initializes the form's start time.
     *
     * This method is called when the form is initially displayed (during the onAfterInitiate event).
     * It records the current timestamp for later validation of the time elapsed before submission.
     * If the start time is already set, it does not overwrite it. 
     */
    public function onInit()
    {
        $this->setStartTime();
    }

    /**
     * Retrieves the form's start time from the session
     * 
     * @return int|null Returns the stored timestamp when the form was initially displayed
     *                  or null if no timestamp exists in the session
     */
    private function getStartTime()
    {
        return $this->app->getSession()->get($this->sessionProp);
    }
    
    /**
     * Stores the current timestamp in the session
     * 
     * @return void
     */
    private function setStartTime()
    {
        return $this->app->getSession()->set($this->sessionProp, time());
    }

    /**
     * Returns the minimum required time in seconds. Defaults to 2 seconds.
     *
     * @return int
     */
    private function getMinTimeToSubmit()
    {
        $enabled = (bool) $this->getFormRegistry()->get('params.enable_min_time_to_submit', false);

        return $enabled ? (int) $this->getFormRegistry()->get('params.min_time_to_submit', 2) : 0;
    }
}